import streamlit as st
import pandas as pd
import plotly.express as px
import numpy as np
from datetime import datetime
import uuid

import json
import os
from datetime import datetime

# Add this near the beginning of your script, after imports
# Debug: Check saves directory at startup
save_dir = "saves"
if os.path.exists(save_dir):
    st.sidebar.write(f"Save directory exists: {save_dir}")
    st.sidebar.write(f"Contents: {os.listdir(save_dir)}")
else:
    st.sidebar.write(f"Save directory does not exist: {save_dir}")

# Update the main application logic to include the analysis phase

# Add this to your session state initialization at the top
if 'agreements' not in st.session_state:
    st.session_state.agreements = []
if 'disagreements' not in st.session_state:
    st.session_state.disagreements = []
if 'resolutions' not in st.session_state:
    st.session_state.resolutions = []
if 'final_framework' not in st.session_state:
    st.session_state.final_framework = {"summary": "", "status": "In Progress"}
if 'predicates' not in st.session_state:
    st.session_state.predicates = []


# Add this function to create predicate breakdowns
def create_predicate_breakdown():
    st.markdown("## Break Down Arguments into Predicates")
    st.markdown("""
    Break down positions into individual statements (predicates) that can be individually assessed.
    This helps unpack complex arguments and identify specific points of agreement or disagreement.
    """)
    
    # Select a position to break down
    position_names = [p["name"] for p in st.session_state.positions]
    selected_name = st.selectbox(
        "Select a position to break down", 
        position_names,
        key="predicate_position_selector"
    )
    
    if selected_name:
        # Find the selected position
        selected_position = next(p for p in st.session_state.positions if p["name"] == selected_name)
        
        st.markdown(f"### Breaking down: {selected_name}'s position")
        st.markdown(f"**Position statement**: {selected_position['statement']}")
        
        # Display existing predicates for this position
        existing_predicates = [
            p for p in st.session_state.predicates 
            if p["position_id"] == selected_position["id"]
        ]
        
        if existing_predicates:
            st.markdown("#### Existing Predicates:")
            for i, pred in enumerate(existing_predicates):
                col1, col2 = st.columns([5, 1])
                with col1:
                    st.markdown(f"{i+1}. {pred['statement']}")
                with col2:
                    if st.button("Delete", key=f"delete_pred_{pred['id']}"):
                        # Remove this predicate
                        st.session_state.predicates = [
                            p for p in st.session_state.predicates if p["id"] != pred["id"]
                        ]
                        st.success("Predicate deleted")
                        st.rerun()
        
        # Form to add new predicates
        with st.form("add_predicate_form"):
            st.markdown("#### Add New Predicate")
            st.markdown("""
            Extract a single claim or statement from the position.
            Good predicates are:
            - Simple and clear
            - Make a single claim
            - Can be assessed as true or false
            - Do not contain multiple ideas
            """)
            
            new_predicate = st.text_area(
                "Predicate Statement",
                placeholder="Enter a single claim extracted from the position..."
            )
            
            predicate_type = st.selectbox(
                "Predicate Type",
                [
                    "Factual Claim",
                    "Value Statement",
                    "Policy Recommendation",
                    "Causal Claim",
                    "Definition",
                    "Method/Process",
                    "Personal Experience",
                    "Other"
                ]
            )
            
            predicate_notes = st.text_area(
                "Notes (Optional)",
                placeholder="Any additional context or notes about this predicate..."
            )
            
            submitted = st.form_submit_button("Add Predicate")
            if submitted:
                if new_predicate:
                    # Create new predicate
                    pred_id = str(uuid.uuid4())
                    new_pred = {
                        "id": pred_id,
                        "position_id": selected_position["id"],
                        "position_name": selected_name,
                        "statement": new_predicate,
                        "type": predicate_type,
                        "notes": predicate_notes,
                        "timestamp": datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
                        "agreements": [],
                        "disagreements": []
                    }
                    
                    st.session_state.predicates.append(new_pred)
                    st.success("Predicate added successfully!")
                    st.rerun()
                else:
                    st.error("Please enter a predicate statement")

# Add this function to analyze predicates collectively
def analyze_predicates():
    st.markdown("## Analyze Predicates")
    st.markdown("""
    Review all predicates extracted from positions and indicate your agreement or disagreement with each.
    This helps identify specific points of consensus and contention.
    """)
    
    if not st.session_state.predicates:
        st.info("No predicates have been created yet. Break down positions into predicates first.")
        return
    
    # Filter options
    filter_col1, filter_col2 = st.columns(2)
    with filter_col1:
        filter_type = st.multiselect(
            "Filter by Type",
            options=list(set(p["type"] for p in st.session_state.predicates)),
            default=list(set(p["type"] for p in st.session_state.predicates))
        )
    
    with filter_col2:
        filter_position = st.multiselect(
            "Filter by Position",
            options=list(set(p["position_name"] for p in st.session_state.predicates)),
            default=list(set(p["position_name"] for p in st.session_state.predicates))
        )
    
    # Filter predicates based on selections
    filtered_predicates = [
        p for p in st.session_state.predicates
        if p["type"] in filter_type and p["position_name"] in filter_position
    ]
    
    # Get current user name (for tracking agreements/disagreements)
    user_name = st.text_input("Your Name (for recording agreements/disagreements)")
    
    if not user_name:
        st.warning("Please enter your name to record your agreements/disagreements")
    
    # Display predicates for analysis
    for i, predicate in enumerate(filtered_predicates):
        with st.expander(f"{predicate['position_name']}: {predicate['statement'][:100]}...", expanded=False):
            st.markdown(f"**Full Statement**: {predicate['statement']}")
            st.markdown(f"**Type**: {predicate['type']}")
            if predicate['notes']:
                st.markdown(f"**Notes**: {predicate['notes']}")
            
            # Display current agreements/disagreements
            agreement_count = len(predicate.get('agreements', []))
            disagreement_count = len(predicate.get('disagreements', []))
            
            # Show who agrees/disagrees
            agree_col, disagree_col = st.columns(2)
            with agree_col:
                st.markdown(f"**Agreements** ({agreement_count}):")
                for person in predicate.get('agreements', []):
                    st.markdown(f"- {person}")
            
            with disagree_col:
                st.markdown(f"**Disagreements** ({disagreement_count}):")
                for person in predicate.get('disagreements', []):
                    st.markdown(f"- {person}")
            
            # Only show agreement buttons if user provided their name
            if user_name:
                # Agreement buttons
                action_col1, action_col2, action_col3 = st.columns(3)
                
                with action_col1:
                    if st.button("I Agree", key=f"agree_{predicate['id']}"):
                        # Add to agreements if not already there
                        if user_name not in predicate.get('agreements', []):
                            # Initialize list if needed
                            if 'agreements' not in predicate:
                                predicate['agreements'] = []
                            
                            predicate['agreements'].append(user_name)
                            
                            # Remove from disagreements if present
                            if 'disagreements' in predicate and user_name in predicate['disagreements']:
                                predicate['disagreements'].remove(user_name)
                            
                            st.success(f"You've agreed with this predicate")
                            st.rerun()
                
                with action_col2:
                    if st.button("I Disagree", key=f"disagree_{predicate['id']}"):
                        # Add to disagreements if not already there
                        if user_name not in predicate.get('disagreements', []):
                            # Initialize list if needed
                            if 'disagreements' not in predicate:
                                predicate['disagreements'] = []
                            
                            predicate['disagreements'].append(user_name)
                            
                            # Remove from agreements if present
                            if 'agreements' in predicate and user_name in predicate['agreements']:
                                predicate['agreements'].remove(user_name)
                            
                            st.success(f"You've disagreed with this predicate")
                            st.rerun()
                
                with action_col3:
                    if st.button("Reset My Vote", key=f"reset_{predicate['id']}"):
                        # Remove from both lists
                        if 'agreements' in predicate and user_name in predicate['agreements']:
                            predicate['agreements'].remove(user_name)
                        
                        if 'disagreements' in predicate and user_name in predicate['disagreements']:
                            predicate['disagreements'].remove(user_name)
                        
                        st.success(f"Your vote has been reset")
                        st.rerun()


def resolution_phase():
    st.header("Phase III: Resolution")
    st.subheader(f"Thesis: {st.session_state.thesis['statement']}")
    
    if not st.session_state.disagreements and not st.session_state.agreements:
        st.warning("No agreements or disagreements have been identified. Please complete the Analysis phase first.")
        if st.button("Go to Analysis Phase", key="go_to_analysis"):
            st.session_state.current_page = "analysis"
            st.rerun()
        return
    
    # Initialize resolution data structure if it doesn't exist
    if 'resolutions' not in st.session_state:
        st.session_state.resolutions = []
    if 'final_framework' not in st.session_state:
        st.session_state.final_framework = {"summary": "", "status": "In Progress"}
    
    # SECTION 1: Overview of Agreements and Disagreements
    col1, col2 = st.columns(2)
    
    with col1:
        st.markdown("## Agreements")
        if st.session_state.agreements:
            for agreement in st.session_state.agreements:
                st.markdown(f"✅ **{agreement['title']}** ({len(agreement['positions'])} positions)")
        else:
            st.info("No agreements identified")
    
    with col2:
        st.markdown("## Key Disagreements")
        if st.session_state.disagreements:
            # Sort disagreements by importance
            sorted_disagreements = sorted(
                st.session_state.disagreements, 
                key=lambda x: x["importance"], 
                reverse=True
            )
            for disagreement in sorted_disagreements:
                importance_emoji = "🔥" if disagreement["importance"] >= 7 else "⚠️" if disagreement["importance"] >= 4 else "ℹ️"
                st.markdown(f"{importance_emoji} **{disagreement['title']}** ({disagreement['type']})")
        else:
            st.info("No disagreements identified")
    
    # SECTION 2: Resolve Important Disagreements
    st.markdown("## Resolve Disagreements")
    
    if not st.session_state.disagreements:
        st.info("No disagreements to resolve")
    else:
        # Focus on highest importance disagreements first
        sorted_disagreements = sorted(
            st.session_state.disagreements, 
            key=lambda x: x["importance"], 
            reverse=True
        )
        
        selected_disagreement_title = st.selectbox(
            "Select a disagreement to resolve:",
            [f"{d['title']} (Importance: {d['importance']}/10)" for d in sorted_disagreements]
        )
        
        selected_idx = next(
            i for i, d in enumerate(sorted_disagreements) 
            if f"{d['title']} (Importance: {d['importance']}/10)" == selected_disagreement_title
        )
        disagreement = sorted_disagreements[selected_idx]
        
        st.markdown(f"### {disagreement['title']}")
        st.markdown(f"**Type**: {disagreement['type']}")
        st.markdown(f"**Description**: {disagreement['description']}")
        
        # Display opposing viewpoints
        col1, col2 = st.columns(2)
        with col1:
            st.markdown("**Side A**:")
            for pos in disagreement['side_a']:
                st.markdown(f"- {pos}")
        
        with col2:
            st.markdown("**Side B**:")
            for pos in disagreement['side_b']:
                st.markdown(f"- {pos}")
        
        # Resolution approach based on disagreement type
        st.markdown("### Resolution Approach")
        
        if "Factual" in disagreement['type']:
            st.info("""
            This is a factual disagreement. Resolution should focus on:
            1. Identifying testable claims
            2. Gathering evidence
            3. Evaluating sources
            4. Finding consensus on methodology
            """)
            
            resolution_approach = st.selectbox(
                "Resolution Method", 
                [
                    "Evidence Collection",
                    "Source Evaluation",
                    "Expert Testimony",
                    "Experiment/Testing",
                    "Statistical Analysis"
                ]
            )
            
        elif "Subjective" in disagreement['type'] or "Opinion" in disagreement['type']:
            st.info("""
            This is a subjective or opinion-based disagreement. Resolution should focus on:
            1. Acknowledging difference of perspective
            2. Finding common values
            3. Understanding underlying motivations
            4. Accepting plurality of views where necessary
            """)
            
            resolution_approach = st.selectbox(
                "Resolution Method", 
                [
                    "Values Clarification",
                    "Perspective Taking",
                    "Accepting Plurality",
                    "Finding Middle Ground",
                    "Agreeing to Disagree"
                ]
            )
            
        elif "Definitional" in disagreement['type'] or "Conceptual" in disagreement['type']:
            st.info("""
            This is a definitional or conceptual disagreement. Resolution should focus on:
            1. Clarifying terms
            2. Finding agreed definitions
            3. Separating substantive from semantic disagreements
            """)
            
            resolution_approach = st.selectbox(
                "Resolution Method", 
                [
                    "Term Clarification",
                    "Creating Shared Definitions",
                    "Conceptual Analysis",
                    "Distinguishing Concepts"
                ]
            )
            
        elif "Value" in disagreement['type']:
            st.info("""
            This is a value-based disagreement. Resolution should focus on:
            1. Identifying underlying values
            2. Finding shared higher-order values
            3. Respecting value pluralism
            4. Seeking compromise solutions
            """)
            
            resolution_approach = st.selectbox(
                "Resolution Method", 
                [
                    "Value Hierarchy Analysis",
                    "Finding Shared Moral Foundations",
                    "Compromise Solution",
                    "Principled Pluralism"
                ]
            )
            
        elif "Unfalsifiable" in disagreement['type']:
            st.info("""
            This involves unfalsifiable claims. Resolution should focus on:
            1. Identifying testable implications
            2. Moving to practical consequences
            3. Acknowledging limits of empirical resolution
            """)
            
            resolution_approach = st.selectbox(
                "Resolution Method", 
                [
                    "Focus on Practical Implications",
                    "Identify Testable Components",
                    "Separate from Policy Decisions",
                    "Acknowledge Uncertainty"
                ]
            )
        
        # Form to add resolution
        with st.form("resolution_form"):
            resolution_notes = st.text_area(
                "Resolution Notes",
                placeholder="Describe how this disagreement was or could be resolved..."
            )
            
            resolution_status = st.selectbox(
                "Status",
                [
                    "Resolved by Consensus",
                    "Partially Resolved",
                    "Acknowledged Difference",
                    "Requires Further Input",
                    "Unresolvable"
                ]
            )
            
            submitted = st.form_submit_button("Add Resolution")
            if submitted:
                if resolution_notes:
                    # Get the actual disagreement from the session state
                    original_idx = next(
                        i for i, d in enumerate(st.session_state.disagreements) 
                        if d["id"] == disagreement["id"]
                    )
                    
                    # Create resolution record
                    new_resolution = {
                        "id": str(uuid.uuid4()),
                        "disagreement_id": disagreement["id"],
                        "disagreement_title": disagreement["title"],
                        "approach": resolution_approach,
                        "notes": resolution_notes,
                        "status": resolution_status,
                        "timestamp": datetime.now().strftime("%Y-%m-%d %H:%M:%S")
                    }
                    
                    # Add resolution to state
                    st.session_state.resolutions.append(new_resolution)
                    
                    # Update the disagreement to mark it has a resolution
                    st.session_state.disagreements[original_idx]["has_resolution"] = True
                    st.session_state.disagreements[original_idx]["resolution_status"] = resolution_status
                    
                    st.success("Resolution added successfully!")
                    st.rerun()
                else:
                    st.error("Please enter resolution notes")
    
    # SECTION 3: Display existing resolutions
    st.markdown("## Existing Resolutions")
    
    if not st.session_state.resolutions:
        st.info("No resolutions have been added yet")
    else:
        for i, resolution in enumerate(st.session_state.resolutions):
            status_emoji = "✅" if "Resolved" in resolution["status"] else "⚠️" if "Partially" in resolution["status"] else "❓"
            with st.expander(f"{status_emoji} {resolution['disagreement_title']} ({resolution['status']})"):
                st.markdown(f"**Approach**: {resolution['approach']}")
                st.markdown(f"**Notes**: {resolution['notes']}")
                st.markdown(f"**Added**: {resolution['timestamp']}")
                
                # Option to delete this resolution
                if st.button("Delete This Resolution", key=f"delete_resolution_{i}"):
                    # Find the disagreement and update its resolution status
                    for j, d in enumerate(st.session_state.disagreements):
                        if d["id"] == resolution["disagreement_id"]:
                            st.session_state.disagreements[j].pop("has_resolution", None)
                            st.session_state.disagreements[j].pop("resolution_status", None)
                    
                    # Remove the resolution
                    st.session_state.resolutions.pop(i)
                    st.success("Resolution deleted")
                    st.rerun()
    
    # SECTION 4: Final Truth Framework
    st.markdown("## Final Truth Framework")
    st.markdown("Synthesize agreements and resolved disagreements into a final truth framework.")
    
    # Count how many disagreements have resolutions
    resolved_count = sum(1 for d in st.session_state.disagreements if d.get("has_resolution"))
    total_disagreements = len(st.session_state.disagreements)
    resolution_percentage = (resolved_count / total_disagreements * 100) if total_disagreements > 0 else 0
    
    st.progress(resolution_percentage / 100)
    st.markdown(f"**Resolution Progress**: {resolved_count}/{total_disagreements} disagreements addressed ({resolution_percentage:.1f}%)")
    
    with st.form("final_framework_form"):
        framework_summary = st.text_area(
            "Truth Framework Summary", 
            value=st.session_state.final_framework["summary"],
            height=200,
            help="Synthesize the current state of understanding, including agreements and resolved disagreements"
        )
        
        framework_status = st.selectbox(
            "Framework Status",
            [
                "In Progress",
                "Provisional Consensus",
                "Strong Consensus",
                "Partial Consensus with Remaining Differences",
                "Multiple Valid Perspectives Identified",
                "Fundamental Disagreement Remains"
            ],
            index=["In Progress", "Provisional Consensus", "Strong Consensus", 
                   "Partial Consensus with Remaining Differences", 
                   "Multiple Valid Perspectives Identified", 
                   "Fundamental Disagreement Remains"].index(st.session_state.final_framework["status"])
        )
        
        submitted = st.form_submit_button("Update Truth Framework")
        if submitted:
            st.session_state.final_framework = {
                "summary": framework_summary,
                "status": framework_status,
                "last_updated": datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
                "agreements": len(st.session_state.agreements),
                "disagreements": total_disagreements,
                "resolved": resolved_count
            }
            st.success("Truth framework updated!")
            st.rerun()
    
    # Navigation buttons
    col1, col2 = st.columns(2)
    with col1:
        if st.button("Back to Analysis", key="back_to_analysis"):
            st.session_state.current_page = "analysis"
            st.rerun()
    with col2:
        if st.button("Generate Final Report", key="generate_report"):
            st.session_state.current_page = "report"
            st.rerun()


def analyze_positions():
    st.header("Phase II: Agreement and Disagreement Analysis")
    st.subheader(f"Thesis: {st.session_state.thesis['statement']}")
    
    if not st.session_state.positions:
        st.warning("No positions have been submitted yet. Please submit positions first.")
        if st.button("Go to Position Submission", key="go_to_position_submission"):
            st.session_state.current_page = "position_submission"
            st.rerun()
        return
    
    # Initialize agreement and disagreement structures if they don't exist
    if 'agreements' not in st.session_state:
        st.session_state.agreements = []
    if 'disagreements' not in st.session_state:
        st.session_state.disagreements = []
    if 'predicates' not in st.session_state:
        st.session_state.predicates = []

    # Create tabs for different analysis activities
    analysis_tab, predicate_tab, predicate_analysis_tab = st.tabs([
        "Agreement & Disagreement", 
        "Break Down Arguments", 
        "Predicate Analysis"
    ])

    with analysis_tab:
        # SECTION 1: View all positions for reference
        with st.expander("View All Positions", expanded=False):
            for i, position in enumerate(st.session_state.positions):
                st.markdown(f"**{position['name']}**: {position['statement']}")
                if i < len(st.session_state.positions) - 1:
                    st.divider()
        
        # SECTION 2: Identify Areas of Agreement
        st.markdown("## Areas of Agreement")
        st.markdown("Identify key points where multiple positions share common ground.")
        
        # Form to add a new agreement
        with st.form("add_agreement_form"):
            agreement_title = st.text_input("Agreement Title", 
                                        placeholder="E.g., 'Everyone agrees that inequality is a problem'")
            
            agreement_description = st.text_area("Detailed Description", 
                                            placeholder="Explain the common ground in detail...")
            
            # Multi-select which positions this agreement encompasses
            position_names = [p["name"] for p in st.session_state.positions]
            related_positions = st.multiselect("Related Positions", 
                                            options=position_names,
                                            help="Select all positions that share this agreement")
            
            # Submit button
            submitted = st.form_submit_button("Add Agreement")
            if submitted:
                if agreement_title and agreement_description and related_positions:
                    new_agreement = {
                        "id": str(uuid.uuid4()),
                        "title": agreement_title,
                        "description": agreement_description,
                        "positions": related_positions,
                        "timestamp": datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
                    }
                    st.session_state.agreements.append(new_agreement)
                    st.success("Agreement added successfully!")
                    st.rerun()
                else:
                    st.error("Please fill in all fields")
        
        # Display existing agreements
        if st.session_state.agreements:
            st.markdown("### Existing Areas of Agreement")
            for i, agreement in enumerate(st.session_state.agreements):
                with st.expander(f"{agreement['title']} ({len(agreement['positions'])} positions)"):
                    st.markdown(f"**Description**: {agreement['description']}")
                    st.markdown("**Related Positions**:")
                    for pos in agreement['positions']:
                        st.markdown(f"- {pos}")
                    
                    # Option to delete this agreement
                    if st.button("Delete This Agreement", key=f"delete_agreement_{i}"):
                        st.session_state.agreements.pop(i)
                        st.success("Agreement deleted")
                        st.rerun()
        else:
            st.info("No areas of agreement identified yet")
        
        # SECTION 3: Identify Areas of Disagreement
        st.markdown("## Areas of Disagreement")
        st.markdown("Identify key points where positions diverge.")
        
        # Form to add a new disagreement
        with st.form("add_disagreement_form"):
            disagreement_title = st.text_input("Disagreement Title", 
                                            placeholder="E.g., 'Views differ on the cause of inequality'")
            
            disagreement_description = st.text_area("Detailed Description", 
                                                placeholder="Explain the nature of the disagreement...")
            
            # For disagreements, we want to categorize them
            disagreement_type = st.selectbox("Type of Disagreement", [
                "Factual (Publicly Testable)",
                "Subjective/Opinion-Based",
                "Definitional/Conceptual",
                "Value-Based",
                "Unfalsifiable Claim"
            ])
            
            # Assess importance
            importance = st.slider("Importance to Thesis", 
                                min_value=1, max_value=10, value=5,
                                help="How central is this disagreement to the main thesis?")
            
            # Multi-select which positions are on each side
            opposing_views = st.text_area("Opposing Viewpoints", 
                                        placeholder="Summarize the different perspectives on this issue...")
            
            position_names = [p["name"] for p in st.session_state.positions]
            side_a_positions = st.multiselect("Positions on Side A", 
                                            options=position_names,
                                            help="Select positions that take one side of the disagreement")
            
            side_b_positions = st.multiselect("Positions on Side B", 
                                            options=position_names,
                                            help="Select positions that take the opposing side")
            
            # Submit button
            submitted = st.form_submit_button("Add Disagreement")
            if submitted:
                if (disagreement_title and disagreement_description and 
                    side_a_positions and side_b_positions and 
                    not any(pos in side_b_positions for pos in side_a_positions)):
                    
                    new_disagreement = {
                        "id": str(uuid.uuid4()),
                        "title": disagreement_title,
                        "description": disagreement_description,
                        "type": disagreement_type,
                        "importance": importance,
                        "opposing_views": opposing_views,
                        "side_a": side_a_positions,
                        "side_b": side_b_positions,
                        "timestamp": datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
                    }
                    st.session_state.disagreements.append(new_disagreement)
                    st.success("Disagreement added successfully!")
                    st.rerun()
                else:
                    if any(pos in side_b_positions for pos in side_a_positions):
                        st.error("A position cannot be on both sides of a disagreement")
                    else:
                        st.error("Please fill in all required fields")
        
        # Display existing disagreements
        if st.session_state.disagreements:
            st.markdown("### Existing Areas of Disagreement")
            for i, disagreement in enumerate(st.session_state.disagreements):
                importance_emoji = "🔥" if disagreement["importance"] >= 7 else "⚠️" if disagreement["importance"] >= 4 else "ℹ️"
                with st.expander(f"{importance_emoji} {disagreement['title']} ({disagreement['type']})"):
                    st.markdown(f"**Description**: {disagreement['description']}")
                    st.markdown(f"**Importance**: {disagreement['importance']}/10")
                    st.markdown(f"**Type**: {disagreement['type']}")
                    
                    st.markdown("**Opposing Viewpoints**:")
                    st.markdown(disagreement['opposing_views'])
                    
                    col1, col2 = st.columns(2)
                    with col1:
                        st.markdown("**Side A**:")
                        for pos in disagreement['side_a']:
                            st.markdown(f"- {pos}")
                    
                    with col2:
                        st.markdown("**Side B**:")
                        for pos in disagreement['side_b']:
                            st.markdown(f"- {pos}")
                    
                    # Option to delete this disagreement
                    if st.button("Delete This Disagreement", key=f"delete_disagreement_{i}"):
                        st.session_state.disagreements.pop(i)
                        st.success("Disagreement deleted")
                        st.rerun()
        else:
            st.info("No areas of disagreement identified yet")

    with predicate_tab:
        create_predicate_breakdown()
    
    with predicate_analysis_tab:
        analyze_predicates()


    # Navigation buttons
    col1, col2 = st.columns(2)
    with col1:
        if st.button("Back to Argument Map", key="back_to_map"):
            st.session_state.current_page = "argument_map"
            st.rerun()
    with col2:
        if st.button("Continue to Resolution Phase", key="to_resolution"):
            st.session_state.current_page = "resolution"
            st.rerun()

    # Add this at the end of your analyze_positions function, if not already there
    st.markdown("---")
    if st.button("Proceed to Resolution Phase", key="proceed_to_resolution"):
        st.session_state.current_page = "resolution"
        st.rerun()


def save_state_to_file(filename=None):
    """Save the current session state to a JSON file"""
    if filename is None or filename.strip() == "":
        # Generate a default filename based on the thesis title and timestamp
        thesis_slug = st.session_state.thesis["statement"][:20].replace(" ", "_").lower()
        timestamp = datetime.now().strftime("%Y%m%d_%H%M%S")
        filename = f"{thesis_slug}_{timestamp}.json"
    else:
        # Ensure filename has .json extension
        if not filename.endswith('.json'):
            filename += '.json'
    
    # Create comprehensive state to save
    state_to_save = {
        # Phase I data
        "thesis": st.session_state.thesis,
        "positions": st.session_state.positions,
        # Phase II data
        "agreements": st.session_state.agreements if "agreements" in st.session_state else [],
        "disagreements": st.session_state.disagreements if "disagreements" in st.session_state else [],
        "predicates": st.session_state.predicates if "predicates" in st.session_state else [],
        # Phase III data
        "resolutions": st.session_state.resolutions if "resolutions" in st.session_state else [],
        "final_framework": st.session_state.final_framework if "final_framework" in st.session_state else {"summary": "", "status": "In Progress"},
        # Metadata
        "save_date": datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
        "current_page": st.session_state.current_page
    }
    
    try:
        # Create a saves directory if it doesn't exist
        os.makedirs("saves", exist_ok=True)
        
        filepath = os.path.join("saves", filename)
        with open(filepath, "w") as f:
            json.dump(state_to_save, f, indent=2)
        return filename
    except Exception as e:
        st.error(f"Error saving state: {e}")
        return None

def load_state_from_file(filename):
    """Load a saved state from a JSON file"""
    try:
        filepath = os.path.join("saves", filename)
        
        if not os.path.exists(filepath):
            st.error(f"File not found: {filepath}")
            return False
        
        with open(filepath, "r") as f:
            loaded_state = json.load(f)
        
        # Update session state with loaded values
        # Phase I data
        st.session_state.thesis = loaded_state["thesis"]
        st.session_state.positions = loaded_state["positions"]
        
        # Phase II data
        if "agreements" in loaded_state:
            st.session_state.agreements = loaded_state["agreements"]
        if "disagreements" in loaded_state:
            st.session_state.disagreements = loaded_state["disagreements"]
        if "predicates" in loaded_state:
            st.session_state.predicates = loaded_state["predicates"]
           
        # Phase III data
        if "resolutions" in loaded_state:
            st.session_state.resolutions = loaded_state["resolutions"]
        if "final_framework" in loaded_state:
            st.session_state.final_framework = loaded_state["final_framework"]
            
        # Restore current page if saved
        if "current_page" in loaded_state:
            st.session_state.current_page = loaded_state["current_page"]
        
        return True
    except Exception as e:
        st.error(f"Error loading state: {str(e)}")
        return False

def get_available_saves():
    """Get a list of available save files"""
    try:
        save_dir = "saves"
        if not os.path.exists(save_dir):
            os.makedirs(save_dir, exist_ok=True)
            return []
        
        files = [f for f in os.listdir(save_dir) if f.endswith(".json")]
        return files
    except Exception as e:
        st.error(f"Error listing save files: {str(e)}")
        return []


# Initialize session state variables if they don't exist
if 'thesis' not in st.session_state:
    st.session_state.thesis = {"statement": "", "purpose": ""}
if 'positions' not in st.session_state:
    st.session_state.positions = []
if 'current_page' not in st.session_state:
    st.session_state.current_page = "thesis_creation"

def create_thesis():
    st.header("Create a Thesis Statement")
    
    st.session_state.thesis["statement"] = st.text_area(
        "Thesis Statement", 
        st.session_state.thesis["statement"], 
        help="What is the proposition being examined?"
    )
    
    # Preset purpose options
    purpose_presets = [
        "Select a preset purpose (optional)",
        "Truth seeking (factual)",
        "Serious policy discussion",
        "Philosophy/Religion discussion", 
        "Socializing/Casual conversation", 
        "Fun debate", 
        "Education", 
        "Brainstorming/Idea generation", 
        "Personal views sharing"
    ]
    
    purpose_selection = st.selectbox(
        "Purpose Presets", 
        purpose_presets,
        index=0,
        help="Choose a common purpose or enter your own below"
    )
    
    # If a preset was selected (not the first option), update the text area
    if purpose_selection != purpose_presets[0]:
        st.session_state.thesis["purpose"] = purpose_selection
    
    # Allow custom purpose entry
    st.session_state.thesis["purpose"] = st.text_area(
        "Purpose", 
        st.session_state.thesis["purpose"], 
        help="Why are we discussing this? What do we hope to accomplish?"
    )
    
    if st.button("Submit Thesis", key="submit_thesis"):
        if st.session_state.thesis["statement"] and st.session_state.thesis["purpose"]:
            st.session_state.current_page = "position_submission"
            st.rerun()
        else:
            st.error("Please provide both a thesis statement and purpose.")

def submit_position():
    st.header(f"Thesis: {st.session_state.thesis['statement']}")
    st.subheader(f"Purpose: {st.session_state.thesis['purpose']}")
    
    st.markdown("### Submit Your Position")
    
    position = {
        "id": str(uuid.uuid4()),
        "name": st.text_input("Your Name"),
        "statement": st.text_area("Your Position", help="Clearly state your position on this thesis"),
        "reasoning": st.text_area("Your Reasoning", help="Explain why you hold this position"),
        "timestamp": datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
        "dimensions": {}
    }
    
    # Optional: Allow users to position themselves on key dimensions
    st.markdown("### Position yourself on these dimensions (optional)")
    
    dimensions = ["Progressive-Conservative", "Individual-Collective", "Practical-Theoretical"]
    for dim in dimensions:
        parts = dim.split("-")
        position["dimensions"][dim] = st.slider(
            dim, 
            min_value=1, 
            max_value=100, 
            value=50,
            help=f"1 = Most {parts[0]}, 100 = Most {parts[1]}"
        )
    
    if st.button("Submit Position", key="submit_position"):
        if position["name"] and position["statement"]:
            st.session_state.positions.append(position)
            st.success("Position submitted!")
            st.session_state.current_page = "argument_map"
            st.rerun()
        else:
            st.error("Please provide both your name and position statement.")
    
    if st.button("Skip to Argument Map", key="skip_to_argument_map"):
        st.session_state.current_page = "argument_map"
        st.rerun()

def display_argument_map():
    st.header(f"Argument Map: {st.session_state.thesis['statement']}")
    
    if not st.session_state.positions:
        st.warning("No positions have been submitted yet.")
        if st.button("Submit a Position", key="submit_a_position"):
            st.session_state.current_page = "position_submission"
            st.rerun()
    
    # Display all positions
    st.markdown("### Current Positions")
    for i, position in enumerate(st.session_state.positions):
        with st.expander(f"{position['name']}'s Position"):
            st.markdown(f"**Statement**: {position['statement']}")
            st.markdown(f"**Reasoning**: {position['reasoning']}")
            st.markdown(f"**Submitted**: {position['timestamp']}")
            
            # Out of scope voting
            col1, col2 = st.columns([3, 1])
            with col2:
                if st.button("Flag as Out of Scope", key=f"flag_{i}"):
                    st.session_state.positions[i]["flagged"] = True
                    st.rerun()
            
            if position.get("flagged", False):
                st.warning("This position has been flagged as potentially out of scope")
    
    # Visualization of positions based on dimensions
    st.markdown("### Position Visualization")
    
    # Create DataFrame for plotting
    plot_data = []
    for position in st.session_state.positions:
        if position["dimensions"]:  # Skip if no dimensions data
            plot_data.append({
                "name": position["name"],
                "dim1": position["dimensions"].get("Progressive-Conservative", 50),
                "dim2": position["dimensions"].get("Individual-Collective", 50)
            })
    
    if plot_data:
        df = pd.DataFrame(plot_data)
        fig = px.scatter(
            df, x="dim1", y="dim2", text="name",
            labels={"dim1": "Progressive-Conservative", "dim2": "Individual-Collective"},
            title="Position Map"
        )
        st.plotly_chart(fig)
    else:
        st.info("Not enough dimension data to create visualization")
    
    # Edit position section
    st.markdown("### Edit Your Position")
    name_to_edit = st.selectbox("Select your name to edit position", 
                               [p["name"] for p in st.session_state.positions])
    
    if name_to_edit:
        position_index = next((i for i, p in enumerate(st.session_state.positions) 
                              if p["name"] == name_to_edit), None)
        
        if position_index is not None:
            position = st.session_state.positions[position_index]
            new_statement = st.text_area("Update Your Position", position["statement"])
            new_reasoning = st.text_area("Update Your Reasoning", position["reasoning"])
            
            if st.button("Update Position", key="update_position"):
                st.session_state.positions[position_index]["statement"] = new_statement
                st.session_state.positions[position_index]["reasoning"] = new_reasoning
                st.session_state.positions[position_index]["timestamp"] = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
                st.success("Position updated!")
                st.rerun()
    
    # Submit new position button
    if st.button("Submit a New Position", key="submit_new_position"):
        st.session_state.current_page = "position_submission"
        st.rerun()

    # Add this at the end of your display_argument_map function
    st.markdown("---")
    if st.button("Proceed to Analysis Phase", key="proceed_to_analysis"):
        st.session_state.current_page = "analysis"
        st.rerun()        
    return


# Main application logic
st.title("Truth-Finding Framework: Phase I, II, and III")

# Navigation based on current page
if st.session_state.current_page == "thesis_creation":
    create_thesis()
elif st.session_state.current_page == "position_submission":
    submit_position()
elif st.session_state.current_page == "argument_map":
    display_argument_map()
elif st.session_state.current_page == "analysis":
    analyze_positions()    
elif st.session_state.current_page == "resolution":
    resolution_phase()    

# Simple navigation sidebar
with st.sidebar:
    st.header("Navigation")
    if st.button("Thesis Creation", key="nav_thesis_creation"):
        st.session_state.current_page = "thesis_creation"
        st.rerun()
    if st.button("Submit Position", key="nav_submit_position"):
        if not st.session_state.thesis["statement"]:
            st.error("Please create a thesis first")
        else:
            st.session_state.current_page = "position_submission"
            st.rerun()
    if st.button("View Argument Map", key="nav_argument_map"):
        if not st.session_state.thesis["statement"]:
            st.error("Please create a thesis first")
        else:
            st.session_state.current_page = "argument_map"
            st.rerun()
    if st.button("Analysis Phase", key="nav_analysis"):
        if not st.session_state.positions:
            st.error("Please submit positions first")
        else:
            st.session_state.current_page = "analysis"
            st.rerun()
    if st.button("Resolution Phase", key="nav_resolution"):
        if not st.session_state.agreements and not st.session_state.disagreements:
            st.error("Please complete Analysis phase first")
        else:
            st.session_state.current_page = "resolution"
            st.rerun()

    st.divider()
    st.markdown("### Current Thesis")
    if st.session_state.thesis["statement"]:
        st.write(st.session_state.thesis["statement"])
    else:
        st.write("No thesis created yet")
    
    st.markdown("### Positions Submitted")
    st.write(f"{len(st.session_state.positions)} positions")

# Add this to the sidebar section of your code
    
    st.divider()
    st.markdown("### Save/Load Discussion")
    
    # Save functionality
    save_tab, load_tab = st.tabs(["Save", "Load"])
    
    with save_tab:
        custom_filename = st.text_input("Filename (optional)", 
                                       placeholder="Leave empty for auto-generated name",
                                       key="save_filename")
        
        if st.button("Save Current State", key="save_button"):
            if not st.session_state.thesis["statement"]:
                st.error("Cannot save: No thesis created yet")
            else:
                filename = save_state_to_file(custom_filename if custom_filename else None)
                if filename:
                    st.success(f"Successfully saved to {filename}")
                    # Force refresh of the available saves
                    st.session_state.last_save_time = datetime.now()
    
    with load_tab:
        # Add a refresh button for available saves
        if st.button("Refresh Available Saves", key="refresh_saves"):
            st.session_state.last_save_time = datetime.now()
        
        available_saves = get_available_saves()
        st.write(f"Found {len(available_saves)} saved discussions")  # Debug output
        
        if not available_saves:
            st.info("No saved discussions found. Check if 'saves' directory exists in the same folder as your script.")
        else:
            selected_save = st.selectbox("Select a saved discussion", 
                                        available_saves,
                                        key="load_selectbox")
            
            st.write(f"Selected: {selected_save}")  # Debug output
            
            if st.button("Load Selected Discussion", key="load_button"):
                if load_state_from_file(selected_save):
                    st.success("Discussion loaded successfully!")
                    st.session_state.current_page = "argument_map"  # Go to argument map after loading
                    st.rerun()
                else:
                    st.error("Failed to load discussion. See error above.")